<?php
 
/**
 * Progrecss Plugin: Prints CSS-based progress bars
 * 
 * @file       progrecss/syntax.php
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     'ryan.chappelle' <visuallucho_plusplus@yahoo.es>
 * @brief      Progress bars with CSS only.
 * @version    1.2
 *
 *   This plugin is intended to follow the same purpose as the
 *  "progressbar" plugin already available in the DW site; 
 *  however, "progrecss"'s inner works are more based on 
 *  CSS code by David Anaxagoras by this URL:
 *   http://www.davidanaxagoras.com/2005/04/16/track-your-progress-or-lack-thereof/
 *
 *   With this plugin, you can create progress bars in your wiki pages, 
 *  using CSS styles only. The PHP side allows for some syntax and parameters
 *  such as bar caption, total width and style used.
 *
 *   For a live demo point your browser to:    http://informatica.temuco.udelmar.cl/~lmachuca/dokuwiki-lucho/desarrollo/informatica/dokuwiki/plugin-progrecss
 *  (in Spanish, neuter enough for online translator to 
 *  provide good translations)
 *
 *  Greetings.
 *        - ryan.chappelle
 */
 
 
if(!defined('DW_LF')) define('DW_LF',"\n");
 
if(!defined('DOKU_INC')) 
define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) 
define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_progrecss extends DokuWiki_Syntax_Plugin {
 
    /**
     * return some info
     */
    function getInfo(){
        return array(
            'author' => 'ryan.chappelle',
            'email'  => 'visuallucho_plusplus@yahoo.es',
            'date'   => '02/01/2008',
            'name'   => 'Progrecss Plugin',
            'desc'   => 'Prints CSS-based progress bars',
            'url'    => 'http://wiki.splitbrain.org/plugin:progrecss',
        );
    }
 
    /**
     * What kind of syntax are we?
     */
    function getType(){
        return 'substition';
    }
 
    /**
     * What can we Do?
     */
    function getAllowedTypes() { 
        return array('formatting', 'substition', 'disabled'); 
    }
 
    /**
     * Where to sort in?
     */
    function getSort(){
        return 149;
        // note: I just came up with a number. I'm still not sure 
        // how to correctly use this thing.
    }
 
    /**
     * What's our code layout?
     */    
    function getPType(){ 
        return 'normal'; 
    }
 
    /**
     * How do we get to the lexer?
     */
    function connectTo($mode) {
         $p_param= ".*?";
 
/** Probably I have a bug in my PHP implementation (compiled source code) 
 *  which prevents me from using a single regex to catch percentages.
 *  Momentarily I am providing two regexes, they should be fine.
 */
 
         $this->Lexer->addSpecialPattern ("<progrecss [0-9]{1,2}% $p_param/>", $mode, 'plugin_progrecss');
         $this->Lexer->addSpecialPattern ("<progrecss 100% $p_param/>", $mode, 'plugin_progrecss');
 
    }
 
    /**
     * Handle the match
     */
    function handle($match, $state, $pos, &$handler){
 
        /* The syntax is expected as follows:
          ppp% [param1=value1; [param2=value2; [...]]] 
         */
        $data= array();
        $expected_params= array(
          "caption", // the progrecss bar item caption
          "style",   // the CSS style used, as in 'progrecss_<stylename>'
          "width",   // the CSS width value for the block
          "order",   // --- reserved for future use ---
          "marker",  // --- reserved for future use ---
          "pdisplay" // --- reserved for future use ---
          );
        /* OK, so let's strip the markup, taking first the precentage%
           value which is _mandatory_. */
        $match = substr($match,11,-2); // Strip markup
        $match= explode(' ', $match, 2);
        $data['p']= $match[0];
        /* That already done, let's split all other elements, which are 
           expected to be in the 'param=value;' format.*/
        $elems= explode(';',$match[1]);
        foreach ($elems as $pair) {
           list($key,$value)= explode('=', $pair, 2);
           trim($key); trim($value);
           /* we check if the param is valid (exists in 'expected_params'), 
              otherwise we just choose to ignore it.
              (yes, something better could've been done) */
           if  (in_array($key, $expected_params) ) { //accept parameter
              $data[$key]= $value;
           } else { //refuse parameter
           }
        }
        /* At this point, if any of the available parameters was 
           not found, let's apply some sane default. */
        if (empty($data['caption']))   $data['caption']= "Caption";
        if (empty($data['style'])  )   $data['style']= "default";
        if (empty($data['width'])  )   $data['width']= "200px";
 
        $data['style']= "progrecss_". $data['style'];
 
        /* In order to somewhat uniquely IDfy each p-bar, let's mix and 
           spice using some of the available parameters. */
        $temp_ID= "p-bar_";
        if (strlen($data['caption'] > 6)) {
           $IDcap= substr($data['caption'], 6, 0);
        } else {
           $IDcap= "cap". rand(1,9);
           }
        $IDnum= sprintf("w%02ds%02xr%04d", 
                (int)substr($data['width'], 2, 0), 
                (int)substr($data['style'], -1, 1), 
                rand(1000,9999) );
        /* Did I already warn you that this futile effort is NOT guaranteed 
           to generate a fully unique ID? */
 
        $data['id']= $temp_ID. $IDcap. $IDnum;
 
        /* Are we ready yet? */
        return $data;
 
    }  
 
    /**
     * Create output
     */
    function render($mode, &$renderer, $data) {
        if($mode == 'xhtml'){
            $percentage=  $data['p'];
            $id= $data['id'];
            $fullwidth=   $data['width'];
            $caption= $data['caption'];
            $style= $data['style']; 
 
            /* each progrecssbar is enclosed in a DIV of its own, 
               classed according to "style" parameter,
               and IDed in a somewhat-unique manner using both caption
               and random binary toughts. */
            $renderer->doc .= DW_LF. '<div class="'. $style. '" ';
            $renderer->doc .= 'id="'. $id. '">'. DW_LF;
            $renderer->doc .= '  <div class="border" style="width: '. $fullwidth. ';">'. DW_LF;
            /* The next 2-liner contains the actual "intelligence" behind 
               the plugin. The rest is simply "ability".*/
            $renderer->doc .= '  <div class="bar" style="width: '. $percentage. ';">';
            $renderer->doc .= $percentage. '</div>'. DW_LF;
            /* See? A PHP+CSS is as powerful as... */
            $renderer->doc .= '  </div>'. DW_LF;
            $renderer->doc .= '  <span class="caption">'. $caption. '</span>';
            $renderer->doc .= DW_LF;
            $renderer->doc .= '</div>';
            return true;
        }
        /* That's all about rendering that needs to be done, at the moment */
        /* edit 2008-06-01: maybe in the futur I may try some ODT support,
           guess it would be REALLY nice for printed documents.
        */
        return false;
    }
 
}
 

